@@ -14,6 +14,12 @@ import shutil
 import bz2
 import io
 
+# Fallback to find_executable from distutils.spawn is a stopgap for
+# supporting V8 builds, which do not yet support Python 3.
+try:
+  from shutil import which
+except ImportError:
+  from distutils.spawn import find_executable as which
 from distutils.version import StrictVersion
 
 # If not run from node/, cd to node/.
@@ -47,7 +53,7 @@ parser = argparse.ArgumentParser()
 valid_os = ('win', 'mac', 'solaris', 'freebsd', 'openbsd', 'linux',
             'android', 'aix', 'cloudabi')
 valid_arch = ('arm', 'arm64', 'ia32', 'mips', 'mipsel', 'mips64el', 'ppc',
-              'ppc64', 'x64', 'x86', 'x86_64', 's390x', 'riscv64', 'loong64')
+              'ppc64', 'x32','x64', 'x86', 'x86_64', 's390x', 'riscv64')
 valid_arm_float_abi = ('soft', 'softfp', 'hard')
 valid_arm_fpu = ('vfp', 'vfpv3', 'vfpv3-d16', 'neon')
 valid_mips_arch = ('loongson', 'r1', 'r2', 'r6', 'rx')
@@ -62,9 +68,6 @@ shared_optgroup = parser.add_argument_group("Shared libraries",
     "Flags that allows you to control whether you want to build against "
     "built-in dependencies or its shared representations. If necessary, "
     "provide multiple libraries with comma.")
-static_optgroup = parser.add_argument_group("Static libraries",
-    "Flags that allows you to control whether you want to build against "
-    "additional static libraries.")
 intl_optgroup = parser.add_argument_group("Internationalization",
     "Flags that lets you enable i18n features in Node.js as well as which "
     "library you want to build against.")
@@ -192,11 +195,6 @@ parser.add_argument("--openssl-no-asm",
     default=None,
     help="Do not build optimized assembly for OpenSSL")
 
-parser.add_argument('--openssl-fips',
-    action='store',
-    dest='openssl_fips',
-    help='Build OpenSSL using FIPS canister .o file in supplied folder')
-
 parser.add_argument('--openssl-is-fips',
     action='store_true',
     dest='openssl_is_fips',
@@ -421,13 +419,6 @@ shared_optgroup.add_argument('--shared-cares-libpath',
 
 parser.add_argument_group(shared_optgroup)
 
-static_optgroup.add_argument('--static-zoslib-gyp',
-    action='store',
-    dest='static_zoslib_gyp',
-    help='path to zoslib.gyp file for includes and to link to static zoslib libray')
-
-parser.add_argument_group(static_optgroup)
-
 parser.add_argument('--systemtap-includes',
     action='store',
     dest='systemtap_includes',
@@ -650,12 +641,6 @@ parser.add_argument('--without-npm',
     default=None,
     help='do not install the bundled npm (package manager)')
 
-parser.add_argument('--without-corepack',
-    action='store_true',
-    dest='without_corepack',
-    default=None,
-    help='do not install the bundled Corepack')
-
 # Dummy option for backwards compatibility
 parser.add_argument('--without-report',
     action='store_true',
@@ -779,13 +764,6 @@ parser.add_argument('--v8-enable-object-print',
     default=True,
     help='compile V8 with auxiliar functions for native debuggers')
 
-parser.add_argument('--v8-enable-hugepage',
-    action='store_true',
-    dest='v8_enable_hugepage',
-    default=None,
-    help='Enable V8 transparent hugepage support. This feature is only '+
-         'available on Linux platform.')
-
 parser.add_argument('--node-builtin-modules-path',
     action='store',
     dest='node_builtin_modules_path',
@@ -850,7 +828,7 @@ def pkg_config(pkg):
   otherwise (None, None, None, None)"""
   pkg_config = os.environ.get('PKG_CONFIG', 'pkg-config')
   args = []  # Print pkg-config warnings on first round.
-  retval = []
+  retval = ()
   for flag in ['--libs-only-l', '--cflags-only-I',
                '--libs-only-L', '--modversion']:
     args += [flag]
@@ -865,9 +843,9 @@ def pkg_config(pkg):
     except OSError as e:
       if e.errno != errno.ENOENT: raise e  # Unexpected error.
       return (None, None, None, None)  # No pkg-config/pkgconf installed.
-    retval.append(val)
+    retval += (val,)
     args = ['--silence-errors']
-  return tuple(retval)
+  return retval
 
 
 def try_check_compiler(cc, lang):
@@ -880,11 +858,7 @@ def try_check_compiler(cc, lang):
   proc.stdin.write(b'__clang__ __GNUC__ __GNUC_MINOR__ __GNUC_PATCHLEVEL__ '
                    b'__clang_major__ __clang_minor__ __clang_patchlevel__')
 
-  if sys.platform == 'zos':
-    values = (to_utf8(proc.communicate()[0]).split('\n')[-2].split() + ['0'] * 7)[0:7]
-  else:
-    values = (to_utf8(proc.communicate()[0]).split() + ['0'] * 7)[0:7]
-
+  values = (to_utf8(proc.communicate()[0]).split() + ['0'] * 7)[0:7]
   is_clang = values[0] == '1'
   gcc_version = tuple(map(int, values[1:1+3]))
   clang_version = tuple(map(int, values[4:4+3])) if is_clang else None
@@ -1071,8 +1045,6 @@ def is_arm_hard_float_abi():
 def host_arch_cc():
   """Host architecture check using the CC command."""
 
-  if sys.platform.startswith('zos'):
-    return 's390x'
   k = cc_macros(os.environ.get('CC_host'))
 
   matchup = {
@@ -1158,15 +1130,6 @@ def configure_mips(o, target_arch):
   host_byteorder = 'little' if target_arch in ('mipsel', 'mips64el') else 'big'
   o['variables']['v8_host_byteorder'] = host_byteorder
 
-def configure_zos(o):
-  o['variables']['node_static_zoslib'] = b(True)
-  if options.static_zoslib_gyp:
-    # Apply to all Node.js components for now
-    o['variables']['zoslib_include_dir'] = os.path.dirname(options.static_zoslib_gyp) + '/include'
-    o['include_dirs'] += [o['variables']['zoslib_include_dir']]
-  else:
-    raise Exception('--static-zoslib-gyp=<path to zoslib.gyp file> is required.')
-
 def clang_version_ge(version_checked):
   for compiler in [(CC, 'c'), (CXX, 'c++')]:
     ok, is_clang, clang_version, gcc_version = \
@@ -1191,7 +1154,6 @@ def configure_node(o):
     o['variables']['OS'] = 'android'
   o['variables']['node_prefix'] = options.prefix
   o['variables']['node_install_npm'] = b(not options.without_npm)
-  o['variables']['node_install_corepack'] = b(not options.without_corepack)
   o['variables']['debug_node'] = b(options.debug_node)
   o['default_configuration'] = 'Debug' if options.debug else 'Release'
   o['variables']['error_on_warn'] = b(options.error_on_warn)
@@ -1236,8 +1198,6 @@ def configure_node(o):
     configure_arm(o)
   elif target_arch in ('mips', 'mipsel', 'mips64el'):
     configure_mips(o, target_arch)
-  elif sys.platform == 'zos':
-    configure_zos(o)
 
   if flavor == 'aix':
     o['variables']['node_target_type'] = 'static_library'
@@ -1350,8 +1310,6 @@ def configure_node(o):
     shlib_suffix = '%s.dylib'
   elif sys.platform.startswith('aix'):
     shlib_suffix = '%s.a'
-  elif sys.platform.startswith('zos'):
-    shlib_suffix = '%s.x'
   else:
     shlib_suffix = 'so.%s'
   if '%s' in shlib_suffix:
@@ -1442,11 +1400,7 @@ def configure_v8(o):
     o['variables']['test_isolation_mode'] = 'noop'  # Needed by d8.gyp.
   if options.without_bundled_v8 and options.enable_d8:
     raise Exception('--enable-d8 is incompatible with --without-bundled-v8.')
-  if options.static_zoslib_gyp:
-    o['variables']['static_zoslib_gyp'] = options.static_zoslib_gyp
-  if flavor != 'linux' and options.v8_enable_hugepage:
-    raise Exception('--v8-enable-hugepage is supported only on linux.')
-  o['variables']['v8_enable_hugepage'] = 1 if options.v8_enable_hugepage else 0
+
 
 def configure_openssl(o):
   variables = o['variables']
@@ -1455,8 +1409,7 @@ def configure_openssl(o):
   variables['node_shared_ngtcp2'] = b(options.shared_ngtcp2)
   variables['node_shared_nghttp3'] = b(options.shared_nghttp3)
   variables['openssl_is_fips'] = b(options.openssl_is_fips)
-  variables['openssl_fips'] = ''
-  variables['openssl_quic'] = b(True)
+  variables['node_fipsinstall'] = b(False)
 
   if options.openssl_no_asm:
     variables['openssl_no_asm'] = 1
@@ -1468,8 +1421,8 @@ def configure_openssl(o):
       without_ssl_error('--shared-openssl')
     if options.openssl_no_asm:
       without_ssl_error('--openssl-no-asm')
-    if options.openssl_fips:
-      without_ssl_error('--openssl-fips')
+    if options.openssl_is_fips:
+      without_ssl_error('--openssl-is-fips')
     if options.openssl_default_cipher_list:
       without_ssl_error('--openssl-default-cipher-list')
     return
@@ -1509,17 +1462,18 @@ def configure_openssl(o):
   if options.openssl_no_asm and options.shared_openssl:
     error('--openssl-no-asm is incompatible with --shared-openssl')
 
-  if options.openssl_fips or options.openssl_fips == '':
-     error('FIPS is not supported in this version of Node.js')
-
   if options.openssl_is_fips and not options.shared_openssl:
-    error('--openssl-is-fips is only available with --shared-openssl')
-
-  if options.openssl_is_fips:
     o['defines'] += ['OPENSSL_FIPS']
+    variables['node_fipsinstall'] = b(True)
 
   if options.shared_openssl:
-    variables['openssl_quic'] = b(getsharedopensslhasquic.get_has_quic(options.__dict__['shared_openssl_includes']))
+    has_quic = getsharedopensslhasquic.get_has_quic(options.__dict__['shared_openssl_includes'])
+  else:
+    has_quic = getsharedopensslhasquic.get_has_quic('deps/openssl/openssl/include')
+
+  variables['openssl_quic'] = b(has_quic)
+  if has_quic:
+    o['defines'] += ['NODE_OPENSSL_HAS_QUIC']
 
   configure_library('openssl', o)
 
@@ -1849,9 +1803,6 @@ def configure_intl(o):
   elif sys.platform.startswith('aix'):
     icu_config['variables']['icu_asm_ext'] = 'S'
     icu_config['variables']['icu_asm_opts'] = [ '-a', 'xlc' ]
-  elif sys.platform == 'zos':
-    icu_config['variables']['icu_asm_ext'] = 'S'
-    icu_config['variables']['icu_asm_opts'] = [ '-a', 'zos' ]
   else:
     # assume GCC-compatible asm is OK
     icu_config['variables']['icu_asm_ext'] = 'S'
@@ -1901,7 +1852,7 @@ def make_bin_override():
   # sys.executable. This directory will be prefixed to the PATH, so that
   # other tools that shell out to `python` will use the appropriate python
 
-  which_python = shutil.which('python')
+  which_python = which('python')
   if (which_python and
       os.path.realpath(which_python) == os.path.realpath(sys.executable)):
     return
@@ -1971,15 +1922,6 @@ variables = output['variables']
 del output['variables']
 variables['is_debug'] = B(options.debug)
 
-# make_global_settings for special FIPS linking
-# should not be used to compile modules in node-gyp
-config_fips = { 'make_global_settings' : [] }
-if 'make_fips_settings' in output:
-  config_fips['make_global_settings'] = output['make_fips_settings']
-  del output['make_fips_settings']
-  write('config_fips.gypi', do_not_edit +
-        pprint.pformat(config_fips, indent=2) + '\n')
-
 # make_global_settings should be a root level element too
 if 'make_global_settings' in output:
   make_global_settings = output['make_global_settings']
@@ -2053,8 +1995,8 @@ if options.compile_commands_json:
 if bin_override is not None:
   gyp_args += ['-Dpython=' + sys.executable]
 
-# pass the leftover non-whitespace positional arguments to GYP
-gyp_args += [arg for arg in args if not str.isspace(arg)]
+# pass the leftover positional arguments to GYP
+gyp_args += args
 
 if warn.warned and not options.verbose:
   warn('warnings were emitted in the configure phase')
