@@ -88,44 +88,69 @@ const BuiltinMetadata builtin_metadata[] = {BUILTIN_LIST(
 
 }  // namespace
 
-BailoutId Builtins::GetContinuationBailoutId(Name name) {
-  DCHECK(Builtins::KindOf(name) == TFJ || Builtins::KindOf(name) == TFC ||
-         Builtins::KindOf(name) == TFS);
-  return BailoutId(BailoutId::kFirstBuiltinContinuationId + name);
+BytecodeOffset Builtins::GetContinuationBytecodeOffset(Builtin builtin) {
+  DCHECK(Builtins::KindOf(builtin) == TFJ || Builtins::KindOf(builtin) == TFC ||
+         Builtins::KindOf(builtin) == TFS);
+  return BytecodeOffset(BytecodeOffset::kFirstBuiltinContinuationId +
+                        ToInt(builtin));
 }
 
-Builtins::Name Builtins::GetBuiltinFromBailoutId(BailoutId id) {
-  int builtin_index = id.ToInt() - BailoutId::kFirstBuiltinContinuationId;
-  DCHECK(Builtins::KindOf(builtin_index) == TFJ ||
-         Builtins::KindOf(builtin_index) == TFC ||
-         Builtins::KindOf(builtin_index) == TFS);
-  return static_cast<Name>(builtin_index);
+Builtin Builtins::GetBuiltinFromBytecodeOffset(BytecodeOffset id) {
+  Builtin builtin = Builtins::FromInt(
+      id.ToInt() - BytecodeOffset::kFirstBuiltinContinuationId);
+  DCHECK(Builtins::KindOf(builtin) == TFJ || Builtins::KindOf(builtin) == TFC ||
+         Builtins::KindOf(builtin) == TFS);
+  return builtin;
 }
 
 void Builtins::TearDown() { initialized_ = false; }
 
 const char* Builtins::Lookup(Address pc) {
   // Off-heap pc's can be looked up through binary search.
-  Code maybe_builtin = InstructionStream::TryLookupCode(isolate_, pc);
-  if (!maybe_builtin.is_null()) return name(maybe_builtin.builtin_index());
+  Builtin builtin = InstructionStream::TryLookupCode(isolate_, pc);
+  if (Builtins::IsBuiltinId(builtin)) return name(builtin);
 
   // May be called during initialization (disassembler).
-  if (initialized_) {
-    for (int i = 0; i < builtin_count; i++) {
-      if (isolate_->heap()->builtin(i).contains(pc)) return name(i);
-    }
+  if (!initialized_) return nullptr;
+  for (Builtin builtin = Builtins::kFirst; builtin <= Builtins::kLast;
+       ++builtin) {
+    if (code(builtin).contains(isolate_, pc)) return name(builtin);
   }
   return nullptr;
 }
 
+Handle<Code> Builtins::CallFunction(ConvertReceiverMode mode) {
+  switch (mode) {
+    case ConvertReceiverMode::kNullOrUndefined:
+      return code_handle(Builtin::kCallFunction_ReceiverIsNullOrUndefined);
+    case ConvertReceiverMode::kNotNullOrUndefined:
+      return code_handle(Builtin::kCallFunction_ReceiverIsNotNullOrUndefined);
+    case ConvertReceiverMode::kAny:
+      return code_handle(Builtin::kCallFunction_ReceiverIsAny);
+  }
+  UNREACHABLE();
+}
+
+Handle<Code> Builtins::Call(ConvertReceiverMode mode) {
+  switch (mode) {
+    case ConvertReceiverMode::kNullOrUndefined:
+      return code_handle(Builtin::kCall_ReceiverIsNullOrUndefined);
+    case ConvertReceiverMode::kNotNullOrUndefined:
+      return code_handle(Builtin::kCall_ReceiverIsNotNullOrUndefined);
+    case ConvertReceiverMode::kAny:
+      return code_handle(Builtin::kCall_ReceiverIsAny);
+  }
+  UNREACHABLE();
+}
+
 Handle<Code> Builtins::NonPrimitiveToPrimitive(ToPrimitiveHint hint) {
   switch (hint) {
     case ToPrimitiveHint::kDefault:
-      return builtin_handle(kNonPrimitiveToPrimitive_Default);
+      return code_handle(Builtin::kNonPrimitiveToPrimitive_Default);
     case ToPrimitiveHint::kNumber:
-      return builtin_handle(kNonPrimitiveToPrimitive_Number);
+      return code_handle(Builtin::kNonPrimitiveToPrimitive_Number);
     case ToPrimitiveHint::kString:
-      return builtin_handle(kNonPrimitiveToPrimitive_String);
+      return code_handle(Builtin::kNonPrimitiveToPrimitive_String);
   }
   UNREACHABLE();
 }
@@ -133,39 +158,41 @@ Handle<Code> Builtins::NonPrimitiveToPrimitive(ToPrimitiveHint hint) {
 Handle<Code> Builtins::OrdinaryToPrimitive(OrdinaryToPrimitiveHint hint) {
   switch (hint) {
     case OrdinaryToPrimitiveHint::kNumber:
-      return builtin_handle(kOrdinaryToPrimitive_Number);
+      return code_handle(Builtin::kOrdinaryToPrimitive_Number);
     case OrdinaryToPrimitiveHint::kString:
-      return builtin_handle(kOrdinaryToPrimitive_String);
+      return code_handle(Builtin::kOrdinaryToPrimitive_String);
   }
   UNREACHABLE();
 }
 
-void Builtins::set_builtin(int index, Code builtin) {
-  isolate_->heap()->set_builtin(index, builtin);
+void Builtins::set_code(Builtin builtin, Code code) {
+  DCHECK_EQ(builtin, code.builtin_id());
+  isolate_->heap()->set_builtin(builtin, code);
 }
 
-Code Builtins::builtin(int index) { return isolate_->heap()->builtin(index); }
+Code Builtins::code(Builtin builtin_enum) {
+  return isolate_->heap()->builtin(builtin_enum);
+}
 
-Handle<Code> Builtins::builtin_handle(int index) {
-  DCHECK(IsBuiltinId(index));
+Handle<Code> Builtins::code_handle(Builtin builtin) {
   return Handle<Code>(
-      reinterpret_cast<Address*>(isolate_->heap()->builtin_address(index)));
+      reinterpret_cast<Address*>(isolate_->heap()->builtin_address(builtin)));
 }
 
 // static
-int Builtins::GetStackParameterCount(Name name) {
-  DCHECK(Builtins::KindOf(name) == TFJ);
-  return builtin_metadata[name].data.parameter_count;
+int Builtins::GetStackParameterCount(Builtin builtin) {
+  DCHECK(Builtins::KindOf(builtin) == TFJ);
+  return builtin_metadata[ToInt(builtin)].data.parameter_count;
 }
 
 // static
-CallInterfaceDescriptor Builtins::CallInterfaceDescriptorFor(Name name) {
+CallInterfaceDescriptor Builtins::CallInterfaceDescriptorFor(Builtin builtin) {
   CallDescriptors::Key key;
-  switch (name) {
+  switch (builtin) {
 // This macro is deliberately crafted so as to emit very little code,
 // in order to keep binary size of this function under control.
 #define CASE_OTHER(Name, ...)                          \
-  case k##Name: {                                      \
+  case Builtin::k##Name: {                             \
     key = Builtin_##Name##_InterfaceDescriptor::key(); \
     break;                                             \
   }
@@ -173,7 +200,7 @@ CallInterfaceDescriptor Builtins::CallInterfaceDescriptorFor(Name name) {
                  CASE_OTHER, IGNORE_BUILTIN, CASE_OTHER)
 #undef CASE_OTHER
     default:
-      Builtins::Kind kind = Builtins::KindOf(name);
+      Builtins::Kind kind = Builtins::KindOf(builtin);
       DCHECK_NE(BCH, kind);
       if (kind == TFJ || kind == CPP) {
         return JSTrampolineDescriptor{};
@@ -184,20 +211,20 @@ CallInterfaceDescriptor Builtins::CallInterfaceDescriptorFor(Name name) {
 }
 
 // static
-Callable Builtins::CallableFor(Isolate* isolate, Name name) {
-  Handle<Code> code = isolate->builtins()->builtin_handle(name);
-  return Callable{code, CallInterfaceDescriptorFor(name)};
+Callable Builtins::CallableFor(Isolate* isolate, Builtin builtin) {
+  Handle<Code> code = isolate->builtins()->code_handle(builtin);
+  return Callable{code, CallInterfaceDescriptorFor(builtin)};
 }
 
 // static
-bool Builtins::HasJSLinkage(int builtin_index) {
-  Name name = static_cast<Name>(builtin_index);
-  DCHECK_NE(BCH, Builtins::KindOf(name));
-  return CallInterfaceDescriptorFor(name) == JSTrampolineDescriptor{};
+bool Builtins::HasJSLinkage(Builtin builtin) {
+  DCHECK_NE(BCH, Builtins::KindOf(builtin));
+  return CallInterfaceDescriptorFor(builtin) == JSTrampolineDescriptor{};
 }
 
 // static
-const char* Builtins::name(int index) {
+const char* Builtins::name(Builtin builtin) {
+  int index = ToInt(builtin);
   DCHECK(IsBuiltinId(index));
   return builtin_metadata[index].name;
 }
@@ -205,11 +232,12 @@ const char* Builtins::name(int index) {
 void Builtins::PrintBuiltinCode() {
   DCHECK(FLAG_print_builtin_code);
 #ifdef ENABLE_DISASSEMBLER
-  for (int i = 0; i < builtin_count; i++) {
-    const char* builtin_name = name(i);
-    Handle<Code> code = builtin_handle(i);
-    if (PassesFilter(CStrVector(builtin_name),
-                     CStrVector(FLAG_print_builtin_code_filter))) {
+  for (Builtin builtin = Builtins::kFirst; builtin <= Builtins::kLast;
+       ++builtin) {
+    const char* builtin_name = name(builtin);
+    Handle<Code> code = code_handle(builtin);
+    if (PassesFilter(base::CStrVector(builtin_name),
+                     base::CStrVector(FLAG_print_builtin_code_filter))) {
       CodeTracer::Scope trace_scope(isolate_->GetCodeTracer());
       OFStream os(trace_scope.file());
       code->Disassemble(builtin_name, os, isolate_);
@@ -221,56 +249,67 @@ void Builtins::PrintBuiltinCode() {
 
 void Builtins::PrintBuiltinSize() {
   DCHECK(FLAG_print_builtin_size);
-  for (int i = 0; i < builtin_count; i++) {
-    const char* builtin_name = name(i);
-    const char* kind = KindNameOf(i);
-    Code code = builtin(i);
+  for (Builtin builtin = Builtins::kFirst; builtin <= Builtins::kLast;
+       ++builtin) {
+    const char* builtin_name = name(builtin);
+    const char* kind = KindNameOf(builtin);
+    Code code = Builtins::code(builtin);
     PrintF(stdout, "%s Builtin, %s, %d\n", kind, builtin_name,
            code.InstructionSize());
   }
 }
 
 // static
-Address Builtins::CppEntryOf(int index) {
-  DCHECK(Builtins::IsCpp(index));
-  return builtin_metadata[index].data.cpp_entry;
+Address Builtins::CppEntryOf(Builtin builtin) {
+  DCHECK(Builtins::IsCpp(builtin));
+  return builtin_metadata[ToInt(builtin)].data.cpp_entry;
 }
 
 // static
 bool Builtins::IsBuiltin(const Code code) {
-  return Builtins::IsBuiltinId(code.builtin_index());
+  return Builtins::IsBuiltinId(code.builtin_id());
 }
 
 bool Builtins::IsBuiltinHandle(Handle<HeapObject> maybe_code,
-                               int* index) const {
+                               Builtin* builtin) const {
   Heap* heap = isolate_->heap();
   Address handle_location = maybe_code.address();
-  Address start = heap->builtin_address(0);
-  Address end = heap->builtin_address(Builtins::builtin_count);
+  Address end =
+      heap->builtin_address(static_cast<Builtin>(Builtins::kBuiltinCount));
   if (handle_location >= end) return false;
+  Address start = heap->builtin_address(static_cast<Builtin>(0));
   if (handle_location < start) return false;
-  *index = static_cast<int>(handle_location - start) >> kSystemPointerSizeLog2;
-  DCHECK(Builtins::IsBuiltinId(*index));
+  *builtin = FromInt(static_cast<int>(handle_location - start) >>
+                     kSystemPointerSizeLog2);
   return true;
 }
 
 // static
 bool Builtins::IsIsolateIndependentBuiltin(const Code code) {
-  const int builtin_index = code.builtin_index();
-  return Builtins::IsBuiltinId(builtin_index) &&
-         Builtins::IsIsolateIndependent(builtin_index);
+  const Builtin builtin = code.builtin_id();
+  return Builtins::IsBuiltinId(builtin) &&
+         Builtins::IsIsolateIndependent(builtin);
 }
 
 // static
-void Builtins::InitializeBuiltinEntryTable(Isolate* isolate) {
-  EmbeddedData d = EmbeddedData::FromBlob();
-  Address* builtin_entry_table = isolate->builtin_entry_table();
-  for (int i = 0; i < builtin_count; i++) {
-    // TODO(jgruber,chromium:1020986): Remove the CHECK once the linked issue is
-    // resolved.
-    CHECK(Builtins::IsBuiltinId(isolate->heap()->builtin(i).builtin_index()));
+void Builtins::InitializeIsolateDataTables(Isolate* isolate) {
+  EmbeddedData embedded_data = EmbeddedData::FromBlob(isolate);
+  IsolateData* isolate_data = isolate->isolate_data();
+
+  // The entry table.
+  for (Builtin i = Builtins::kFirst; i <= Builtins::kLast; ++i) {
+    DCHECK(Builtins::IsBuiltinId(isolate->heap()->builtin(i).builtin_id()));
     DCHECK(isolate->heap()->builtin(i).is_off_heap_trampoline());
-    builtin_entry_table[i] = d.InstructionStartOfBuiltin(i);
+    isolate_data->builtin_entry_table()[ToInt(i)] =
+        embedded_data.InstructionStartOfBuiltin(i);
+  }
+
+  // T0 tables.
+  for (Builtin i = Builtins::kFirst; i <= Builtins::kLastTier0; ++i) {
+    const int ii = ToInt(i);
+    isolate_data->builtin_tier0_entry_table()[ii] =
+        isolate_data->builtin_entry_table()[ii];
+    isolate_data->builtin_tier0_table()[ii] = isolate_data->builtin_table()[ii];
   }
 }
 
@@ -281,17 +320,17 @@ void Builtins::EmitCodeCreateEvents(Isolate* isolate) {
     return;  // No need to iterate the entire table in this case.
   }
 
-  Address* builtins = isolate->builtins_table();
+  Address* builtins = isolate->builtin_table();
   int i = 0;
   HandleScope scope(isolate);
-  for (; i < kFirstBytecodeHandler; i++) {
+  for (; i < ToInt(Builtin::kFirstBytecodeHandler); i++) {
     Handle<AbstractCode> code(AbstractCode::cast(Object(builtins[i])), isolate);
     PROFILE(isolate, CodeCreateEvent(CodeEventListener::BUILTIN_TAG, code,
-                                     Builtins::name(i)));
+                                     Builtins::name(FromInt(i))));
   }
 
-  STATIC_ASSERT(kLastBytecodeHandlerPlusOne == builtin_count);
-  for (; i < builtin_count; i++) {
+  STATIC_ASSERT(kLastBytecodeHandlerPlusOne == kBuiltinCount);
+  for (; i < kBuiltinCount; i++) {
     Handle<AbstractCode> code(AbstractCode::cast(Object(builtins[i])), isolate);
     interpreter::Bytecode bytecode =
         builtin_metadata[i].data.bytecode_and_scale.bytecode;
@@ -311,7 +350,8 @@ class OffHeapTrampolineGenerator {
  public:
   explicit OffHeapTrampolineGenerator(Isolate* isolate)
       : isolate_(isolate),
-        masm_(isolate, CodeObjectRequired::kYes,
+        masm_(isolate, AssemblerOptions::DefaultForOffHeapTrampoline(isolate),
+              CodeObjectRequired::kYes,
               ExternalAssemblerBuffer(buffer_, kBufferSize)) {}
 
   CodeDesc Generate(Address off_heap_entry, TrampolineType type) {
@@ -323,6 +363,7 @@ class OffHeapTrampolineGenerator {
         masm_.CodeEntry();
         masm_.JumpToInstructionStream(off_heap_entry);
       } else {
+        DCHECK_EQ(type, TrampolineType::kAbort);
         masm_.Trap();
       }
     }
@@ -361,7 +402,8 @@ Handle<Code> Builtins::GenerateOffHeapTrampolineFor(
                                              : TrampolineType::kAbort);
 
   return Factory::CodeBuilder(isolate, desc, CodeKind::BUILTIN)
-      .set_read_only_data_container(kind_specfic_flags)
+      .set_kind_specific_flags(kind_specfic_flags)
+      .set_read_only_data_container(!V8_EXTERNAL_CODE_SPACE_BOOL)
       .set_self_reference(generator.CodeObject())
       .set_is_executable(generate_jump_to_instruction_stream)
       .Build();
@@ -382,15 +424,14 @@ Handle<ByteArray> Builtins::GenerateOffHeapTrampolineRelocInfo(
   return reloc_info;
 }
 
-// static
-Builtins::Kind Builtins::KindOf(int index) {
-  DCHECK(IsBuiltinId(index));
-  return builtin_metadata[index].kind;
+Builtins::Kind Builtins::KindOf(Builtin builtin) {
+  DCHECK(IsBuiltinId(builtin));
+  return builtin_metadata[ToInt(builtin)].kind;
 }
 
 // static
-const char* Builtins::KindNameOf(int index) {
-  Kind kind = Builtins::KindOf(index);
+const char* Builtins::KindNameOf(Builtin builtin) {
+  Kind kind = Builtins::KindOf(builtin);
   // clang-format off
   switch (kind) {
     case CPP: return "CPP";
@@ -406,7 +447,9 @@ const char* Builtins::KindNameOf(int index) {
 }
 
 // static
-bool Builtins::IsCpp(int index) { return Builtins::KindOf(index) == CPP; }
+bool Builtins::IsCpp(Builtin builtin) {
+  return Builtins::KindOf(builtin) == CPP;
+}
 
 // static
 bool Builtins::AllowDynamicFunction(Isolate* isolate, Handle<JSFunction> target,
@@ -423,7 +466,7 @@ bool Builtins::AllowDynamicFunction(Isolate* isolate, Handle<JSFunction> target,
 }
 
 // static
-bool Builtins::CodeObjectIsExecutable(int builtin_index) {
+bool Builtins::CodeObjectIsExecutable(Builtin builtin) {
   // If the runtime/optimized code always knows when executing a given builtin
   // that it is a builtin, then that builtin does not need an executable Code
   // object. Such Code objects can go in read_only_space (and can even be
@@ -437,7 +480,7 @@ bool Builtins::CodeObjectIsExecutable(int builtin_index) {
   // TODO(delphick): This is probably too loose but for now Wasm can call any JS
   // linkage builtin via its Code object. Once Wasm is fixed this can either be
   // tighted or removed completely.
-  if (Builtins::KindOf(builtin_index) != BCH && HasJSLinkage(builtin_index)) {
+  if (Builtins::KindOf(builtin) != BCH && HasJSLinkage(builtin)) {
     return true;
   }
 
@@ -446,24 +489,26 @@ bool Builtins::CodeObjectIsExecutable(int builtin_index) {
   // TODO(delphick): Some of these builtins do not fit with the above, but
   // currently cause problems if they're not executable. This list should be
   // pared down as much as possible.
-  switch (builtin_index) {
-    case Builtins::kInterpreterEntryTrampoline:
-    case Builtins::kCompileLazy:
-    case Builtins::kCompileLazyDeoptimizedCode:
-    case Builtins::kCallFunction_ReceiverIsNullOrUndefined:
-    case Builtins::kCallFunction_ReceiverIsNotNullOrUndefined:
-    case Builtins::kCallFunction_ReceiverIsAny:
-    case Builtins::kCallBoundFunction:
-    case Builtins::kCall_ReceiverIsNullOrUndefined:
-    case Builtins::kCall_ReceiverIsNotNullOrUndefined:
-    case Builtins::kCall_ReceiverIsAny:
-    case Builtins::kArgumentsAdaptorTrampoline:
-    case Builtins::kHandleApiCall:
-    case Builtins::kInstantiateAsmJs:
+  switch (builtin) {
+    case Builtin::kInterpreterEntryTrampoline:
+    case Builtin::kCompileLazy:
+    case Builtin::kCompileLazyDeoptimizedCode:
+    case Builtin::kCallFunction_ReceiverIsNullOrUndefined:
+    case Builtin::kCallFunction_ReceiverIsNotNullOrUndefined:
+    case Builtin::kCallFunction_ReceiverIsAny:
+    case Builtin::kCallBoundFunction:
+    case Builtin::kCall_ReceiverIsNullOrUndefined:
+    case Builtin::kCall_ReceiverIsNotNullOrUndefined:
+    case Builtin::kCall_ReceiverIsAny:
+    case Builtin::kHandleApiCall:
+    case Builtin::kInstantiateAsmJs:
+#if V8_ENABLE_WEBASSEMBLY
+    case Builtin::kGenericJSToWasmWrapper:
+#endif  // V8_ENABLE_WEBASSEMBLY
 
     // TODO(delphick): Remove this when calls to it have the trampoline inlined
     // or are converted to use kCallBuiltinPointer.
-    case Builtins::kCEntry_Return1_DontSaveFPRegs_ArgvOnStack_NoBuiltinExit:
+    case Builtin::kCEntry_Return1_DontSaveFPRegs_ArgvOnStack_NoBuiltinExit:
       return true;
     default:
 #if V8_TARGET_ARCH_MIPS || V8_TARGET_ARCH_MIPS64
@@ -477,12 +522,12 @@ bool Builtins::CodeObjectIsExecutable(int builtin_index) {
   }
 }
 
-Builtins::Name ExampleBuiltinForTorqueFunctionPointerType(
+Builtin ExampleBuiltinForTorqueFunctionPointerType(
     size_t function_pointer_type_id) {
   switch (function_pointer_type_id) {
 #define FUNCTION_POINTER_ID_CASE(id, name) \
   case id:                                 \
-    return Builtins::k##name;
+    return Builtin::k##name;
     TORQUE_FUNCTION_POINTER_TYPE_TO_BUILTIN_MAP(FUNCTION_POINTER_ID_CASE)
 #undef FUNCTION_POINTER_ID_CASE
     default:
