@@ -27,7 +27,6 @@ const {
   ObjectDefineProperties,
   ObjectDefineProperty,
   ReflectApply,
-  Symbol,
 } = primordials;
 
 const cares = internalBinding('cares_wrap');
@@ -42,36 +41,10 @@ const {
   Resolver,
   validateHints,
   emitInvalidHostnameWarning,
+  emitTypeCoercionDeprecationWarning,
   getDefaultVerbatim,
   setDefaultResultOrder,
-  errorCodes: dnsErrorCodes,
 } = require('internal/dns/utils');
-const {
-  NODATA,
-  FORMERR,
-  SERVFAIL,
-  NOTFOUND,
-  NOTIMP,
-  REFUSED,
-  BADQUERY,
-  BADNAME,
-  BADFAMILY,
-  BADRESP,
-  CONNREFUSED,
-  TIMEOUT,
-  EOF,
-  FILE,
-  NOMEM,
-  DESTRUCTION,
-  BADSTR,
-  BADFLAGS,
-  NONAME,
-  BADHINTS,
-  NOTINITIALIZED,
-  LOADIPHLPAPI,
-  ADDRGETNETWORKPARAMS,
-  CANCELLED,
-} = dnsErrorCodes;
 const {
   ERR_INVALID_ARG_TYPE,
   ERR_INVALID_ARG_VALUE,
@@ -90,16 +63,6 @@ const {
   QueryReqWrap,
 } = cares;
 
-const kPerfHooksDnsLookupContext = Symbol('kPerfHooksDnsLookupContext');
-const kPerfHooksDnsLookupServiceContext = Symbol('kPerfHooksDnsLookupServiceContext');
-const kPerfHooksDnsLookupResolveContext = Symbol('kPerfHooksDnsLookupResolveContext');
-
-const {
-  hasObserver,
-  startPerf,
-  stopPerf,
-} = require('internal/perf/observe');
-
 const dnsException = errors.dnsException;
 
 let promises = null; // Lazy loaded
@@ -109,9 +72,6 @@ function onlookup(err, addresses) {
     return this.callback(dnsException(err, 'getaddrinfo', this.hostname));
   }
   this.callback(null, addresses[0], this.family || isIP(addresses[0]));
-  if (this[kPerfHooksDnsLookupContext] && hasObserver('dns')) {
-    stopPerf(this, kPerfHooksDnsLookupContext, { detail: { addresses } });
-  }
 }
 
 
@@ -130,9 +90,6 @@ function onlookupall(err, addresses) {
   }
 
   this.callback(null, addresses);
-  if (this[kPerfHooksDnsLookupContext] && hasObserver('dns')) {
-    stopPerf(this, kPerfHooksDnsLookupContext, { detail: { addresses } });
-  }
 }
 
 
@@ -156,15 +113,29 @@ function lookup(hostname, options, callback) {
     validateCallback(callback);
 
     if (options !== null && typeof options === 'object') {
+      if (options.hints != null && typeof options.hints !== 'number') {
+        emitTypeCoercionDeprecationWarning();
+      }
       hints = options.hints >>> 0;
+      if (options.family != null && typeof options.family !== 'number') {
+        emitTypeCoercionDeprecationWarning();
+      }
       family = options.family >>> 0;
+      if (options.all != null && typeof options.all !== 'boolean') {
+        emitTypeCoercionDeprecationWarning();
+      }
       all = options.all === true;
       if (typeof options.verbatim === 'boolean') {
         verbatim = options.verbatim === true;
+      } else if (options.verbatim != null) {
+        emitTypeCoercionDeprecationWarning();
       }
 
       validateHints(hints);
     } else {
+      if (options != null && typeof options !== 'number') {
+        emitTypeCoercionDeprecationWarning();
+      }
       family = options >>> 0;
     }
   }
@@ -205,20 +176,11 @@ function lookup(hostname, options, callback) {
     process.nextTick(callback, dnsException(err, 'getaddrinfo', hostname));
     return {};
   }
-  if (hasObserver('dns')) {
-    const detail = {
-      hostname,
-      family,
-      hints,
-      verbatim,
-    };
-    startPerf(req, kPerfHooksDnsLookupContext, { type: 'dns', name: 'lookup', detail });
-  }
   return req;
 }
 
 ObjectDefineProperty(lookup, customPromisifyArgs,
-                     { __proto__: null, value: ['address', 'family'], enumerable: false });
+                     { value: ['address', 'family'], enumerable: false });
 
 
 function onlookupservice(err, hostname, service) {
@@ -226,9 +188,6 @@ function onlookupservice(err, hostname, service) {
     return this.callback(dnsException(err, 'getnameinfo', this.hostname));
 
   this.callback(null, hostname, service);
-  if (this[kPerfHooksDnsLookupServiceContext] && hasObserver('dns')) {
-    stopPerf(this, kPerfHooksDnsLookupServiceContext, { detail: { hostname, service } });
-  }
 }
 
 
@@ -253,21 +212,11 @@ function lookupService(address, port, callback) {
 
   const err = cares.getnameinfo(req, address, port);
   if (err) throw dnsException(err, 'getnameinfo', address);
-  if (hasObserver('dns')) {
-    startPerf(req, kPerfHooksDnsLookupServiceContext, {
-      type: 'dns',
-      name: 'lookupService',
-      detail: {
-        host: address,
-        port,
-      },
-    });
-  }
   return req;
 }
 
 ObjectDefineProperty(lookupService, customPromisifyArgs,
-                     { __proto__: null, value: ['hostname', 'service'], enumerable: false });
+                     { value: ['hostname', 'service'], enumerable: false });
 
 
 function onresolve(err, result, ttls) {
@@ -277,12 +226,8 @@ function onresolve(err, result, ttls) {
 
   if (err)
     this.callback(dnsException(err, this.bindingName, this.hostname));
-  else {
+  else
     this.callback(null, result);
-    if (this[kPerfHooksDnsLookupResolveContext] && hasObserver('dns')) {
-      stopPerf(this, kPerfHooksDnsLookupResolveContext, { detail: { result } });
-    }
-  }
 }
 
 function resolver(bindingName) {
@@ -304,19 +249,9 @@ function resolver(bindingName) {
     req.ttl = !!(options && options.ttl);
     const err = this._handle[bindingName](req, toASCII(name));
     if (err) throw dnsException(err, bindingName, name);
-    if (hasObserver('dns')) {
-      startPerf(req, kPerfHooksDnsLookupResolveContext, {
-        type: 'dns',
-        name: bindingName,
-        detail: {
-          host: name,
-          ttl: req.ttl,
-        },
-      });
-    }
     return req;
   }
-  ObjectDefineProperty(query, 'name', { __proto__: null, value: bindingName });
+  ObjectDefineProperty(query, 'name', { value: bindingName });
   return query;
 }
 
@@ -379,42 +314,43 @@ module.exports = {
   V4MAPPED: cares.AI_V4MAPPED,
 
   // ERROR CODES
-  NODATA,
-  FORMERR,
-  SERVFAIL,
-  NOTFOUND,
-  NOTIMP,
-  REFUSED,
-  BADQUERY,
-  BADNAME,
-  BADFAMILY,
-  BADRESP,
-  CONNREFUSED,
-  TIMEOUT,
-  EOF,
-  FILE,
-  NOMEM,
-  DESTRUCTION,
-  BADSTR,
-  BADFLAGS,
-  NONAME,
-  BADHINTS,
-  NOTINITIALIZED,
-  LOADIPHLPAPI,
-  ADDRGETNETWORKPARAMS,
-  CANCELLED,
+  NODATA: 'ENODATA',
+  FORMERR: 'EFORMERR',
+  SERVFAIL: 'ESERVFAIL',
+  NOTFOUND: 'ENOTFOUND',
+  NOTIMP: 'ENOTIMP',
+  REFUSED: 'EREFUSED',
+  BADQUERY: 'EBADQUERY',
+  BADNAME: 'EBADNAME',
+  BADFAMILY: 'EBADFAMILY',
+  BADRESP: 'EBADRESP',
+  CONNREFUSED: 'ECONNREFUSED',
+  TIMEOUT: 'ETIMEOUT',
+  EOF: 'EOF',
+  FILE: 'EFILE',
+  NOMEM: 'ENOMEM',
+  DESTRUCTION: 'EDESTRUCTION',
+  BADSTR: 'EBADSTR',
+  BADFLAGS: 'EBADFLAGS',
+  NONAME: 'ENONAME',
+  BADHINTS: 'EBADHINTS',
+  NOTINITIALIZED: 'ENOTINITIALIZED',
+  LOADIPHLPAPI: 'ELOADIPHLPAPI',
+  ADDRGETNETWORKPARAMS: 'EADDRGETNETWORKPARAMS',
+  CANCELLED: 'ECANCELLED'
 };
 
 bindDefaultResolver(module.exports, getDefaultResolver());
 
 ObjectDefineProperties(module.exports, {
   promises: {
-    __proto__: null,
     configurable: true,
     enumerable: true,
     get() {
       if (promises === null) {
         promises = require('internal/dns/promises');
+        promises.setServers = defaultResolverSetServers;
+        promises.setDefaultResultOrder = setDefaultResultOrder;
       }
       return promises;
     }
