@@ -149,11 +149,7 @@ function getStdin() {
   switch (guessHandleType(fd)) {
     case 'TTY':
       const tty = require('tty');
-      stdin = new tty.ReadStream(fd, {
-        highWaterMark: 0,
-        readable: true,
-        writable: false
-      });
+      stdin = new tty.ReadStream(fd);
       break;
 
     case 'FILE':
