@@ -46,7 +46,7 @@ const { Buffer } = require('buffer');
 const destroyImpl = require('internal/streams/destroy');
 
 const {
-  addAbortSignalNoValidate,
+  addAbortSignal,
 } = require('internal/streams/add-abort-signal');
 
 const {
@@ -211,7 +211,6 @@ WritableState.prototype.getBuffer = function getBuffer() {
 };
 
 ObjectDefineProperty(WritableState.prototype, 'bufferedRequestCount', {
-  __proto__: null,
   get() {
     return this.buffered.length - this.bufferedIndex;
   }
@@ -250,8 +249,9 @@ function Writable(options) {
 
     if (typeof options.construct === 'function')
       this._construct = options.construct;
+
     if (options.signal)
-      addAbortSignalNoValidate(options.signal, this);
+      addAbortSignal(options.signal, this);
   }
 
   Stream.call(this, options);
@@ -268,7 +268,6 @@ function Writable(options) {
 }
 
 ObjectDefineProperty(Writable, SymbolHasInstance, {
-  __proto__: null,
   value: function(object) {
     if (FunctionPrototypeSymbolHasInstance(this, object)) return true;
     if (this !== Writable) return false;
@@ -516,12 +515,12 @@ function errorBuffer(state) {
     const { chunk, callback } = state.buffered[n];
     const len = state.objectMode ? 1 : chunk.length;
     state.length -= len;
-    callback(new ERR_STREAM_DESTROYED('write'));
+    callback(state.errored ?? new ERR_STREAM_DESTROYED('write'));
   }
 
   const onfinishCallbacks = state[kOnFinished].splice(0);
   for (let i = 0; i < onfinishCallbacks.length; i++) {
-    onfinishCallbacks[i](new ERR_STREAM_DESTROYED('end'));
+    onfinishCallbacks[i](state.errored ?? new ERR_STREAM_DESTROYED('end'));
   }
 
   resetBuffer(state);
@@ -700,19 +699,15 @@ function callFinal(stream, state) {
         then.call(
           result,
           function() {
-            if (!called) {
-              process.nextTick(onFinish, null);
-            }
+            process.nextTick(onFinish, null);
           },
           function(err) {
-            if (!called) {
-              process.nextTick(onFinish, err);
-            }
+            process.nextTick(onFinish, err);
           });
       }
     }
   } catch (err) {
-    onFinish(err);
+    onFinish(stream, state, err);
   }
 
   state.sync = false;
@@ -774,7 +769,6 @@ function finish(stream, state) {
 ObjectDefineProperties(Writable.prototype, {
 
   destroyed: {
-    __proto__: null,
     get() {
       return this._writableState ? this._writableState.destroyed : false;
     },
@@ -787,7 +781,6 @@ ObjectDefineProperties(Writable.prototype, {
   },
 
   writable: {
-    __proto__: null,
     get() {
       const w = this._writableState;
       // w.writable === false means that this is part of a Duplex stream
@@ -806,35 +799,30 @@ ObjectDefineProperties(Writable.prototype, {
   },
 
   writableFinished: {
-    __proto__: null,
     get() {
       return this._writableState ? this._writableState.finished : false;
     }
   },
 
   writableObjectMode: {
-    __proto__: null,
     get() {
       return this._writableState ? this._writableState.objectMode : false;
     }
   },
 
   writableBuffer: {
-    __proto__: null,
     get() {
       return this._writableState && this._writableState.getBuffer();
     }
   },
 
   writableEnded: {
-    __proto__: null,
     get() {
       return this._writableState ? this._writableState.ending : false;
     }
   },
 
   writableNeedDrain: {
-    __proto__: null,
     get() {
       const wState = this._writableState;
       if (!wState) return false;
@@ -843,37 +831,22 @@ ObjectDefineProperties(Writable.prototype, {
   },
 
   writableHighWaterMark: {
-    __proto__: null,
     get() {
       return this._writableState && this._writableState.highWaterMark;
     }
   },
 
   writableCorked: {
-    __proto__: null,
     get() {
       return this._writableState ? this._writableState.corked : 0;
     }
   },
 
   writableLength: {
-    __proto__: null,
     get() {
       return this._writableState && this._writableState.length;
     }
-  },
-
-  writableAborted: {
-    __proto__: null,
-    enumerable: false,
-    get: function() {
-      return !!(
-        this._writableState.writable !== false &&
-        (this._writableState.destroyed || this._writableState.errored) &&
-        !this._writableState.finished
-      );
-    }
-  },
+  }
 });
 
 const destroy = destroyImpl.destroy;
@@ -899,3 +872,22 @@ Writable.prototype._destroy = function(err, cb) {
 Writable.prototype[EE.captureRejectionSymbol] = function(err) {
   this.destroy(err);
 };
+
+let webStreamsAdapters;
+
+// Lazy to avoid circular references
+function lazyWebStreams() {
+  if (webStreamsAdapters === undefined)
+    webStreamsAdapters = require('internal/webstreams/adapters');
+  return webStreamsAdapters;
+}
+
+Writable.fromWeb = function(writableStream, options) {
+  return lazyWebStreams().newStreamWritableFromWritableStream(
+    writableStream,
+    options);
+};
+
+Writable.toWeb = function(streamWritable) {
+  return lazyWebStreams().newWritableStreamFromStreamWritable(streamWritable);
+};
