@@ -25,27 +25,25 @@
 
 #include "debug_utils-inl.h"
 #include "env-inl.h"
-#include "histogram-inl.h"
 #include "memory_tracker-inl.h"
+#include "histogram-inl.h"
 #include "node_binding.h"
-#include "node_builtins.h"
 #include "node_errors.h"
 #include "node_internals.h"
 #include "node_main_instance.h"
 #include "node_metadata.h"
-#include "node_builtins_env.h"
+#include "node_native_module_env.h"
 #include "node_options-inl.h"
 #include "node_perf.h"
 #include "node_process-inl.h"
 #include "node_report.h"
 #include "node_revert.h"
-#include "node_snapshot_builder.h"
 #include "node_v8_platform-inl.h"
 #include "node_version.h"
 
 #if HAVE_OPENSSL
+#include "allocated_buffer-inl.h"  // Inlined functions needed by node_crypto.h
 #include "node_crypto.h"
-#include <openssl/conf.h>
 #endif
 
 #if defined(NODE_HAVE_I18N_SUPPORT)
@@ -127,6 +125,8 @@
 
 namespace node {
 
+using native_module::NativeModuleEnv;
+
 using v8::EscapableHandleScope;
 using v8::Function;
 using v8::FunctionCallbackInfo;
@@ -162,11 +162,6 @@ PVOID old_vectored_exception_handler;
 struct V8Platform v8_platform;
 }  // namespace per_process
 
-// The section in the OpenSSL configuration file to be loaded.
-const char* conf_section_name = STRINGIFY(NODE_OPENSSL_CONF_NAME);
-
-const char* fips_error_msg = "OpenSSL error when trying to enable FIPS:\n";
-
 #ifdef __POSIX__
 void SignalExit(int signo, siginfo_t* info, void* ucontext) {
   ResetStdio();
@@ -180,8 +175,7 @@ MaybeLocal<Value> ExecuteBootstrapper(Environment* env,
                                       std::vector<Local<Value>>* arguments) {
   EscapableHandleScope scope(env->isolate());
   MaybeLocal<Function> maybe_fn =
-      builtins::BuiltinEnv::LookupAndCompile(env->context(),
-          id, parameters, env);
+      NativeModuleEnv::LookupAndCompile(env->context(), id, parameters, env);
 
   Local<Function> fn;
   if (!maybe_fn.ToLocal(&fn)) {
@@ -282,8 +276,9 @@ static void AtomicsWaitCallback(Isolate::AtomicsWaitEvent event,
 void Environment::InitializeDiagnostics() {
   isolate_->GetHeapProfiler()->AddBuildEmbedderGraphCallback(
       Environment::BuildEmbedderGraph, this);
-  if (heap_snapshot_near_heap_limit_ > 0) {
-    AddHeapSnapshotNearHeapLimitCallback();
+  if (options_->heap_snapshot_near_heap_limit > 0) {
+    isolate_->AddNearHeapLimitCallback(Environment::NearHeapLimitCallback,
+                                       this);
   }
   if (options_->trace_uncaught)
     isolate_->SetCaptureStackTraceForUncaughtExceptions(true);
@@ -311,10 +306,10 @@ MaybeLocal<Value> Environment::BootstrapInternalLoaders() {
       primordials_string()};
   std::vector<Local<Value>> loaders_args = {
       process_object(),
-      NewFunctionTemplate(isolate_, binding::GetLinkedBinding)
+      NewFunctionTemplate(binding::GetLinkedBinding)
           ->GetFunction(context())
           .ToLocalChecked(),
-      NewFunctionTemplate(isolate_, binding::GetInternalBinding)
+      NewFunctionTemplate(binding::GetInternalBinding)
           ->GetFunction(context())
           .ToLocalChecked(),
       primordials()};
@@ -336,7 +331,7 @@ MaybeLocal<Value> Environment::BootstrapInternalLoaders() {
   Local<Value> require =
       loader_exports_obj->Get(context(), require_string()).ToLocalChecked();
   CHECK(require->IsFunction());
-  set_builtin_module_require(require.As<Function>());
+  set_native_module_require(require.As<Function>());
 
   return scope.Escape(loader_exports);
 }
@@ -344,6 +339,11 @@ MaybeLocal<Value> Environment::BootstrapInternalLoaders() {
 MaybeLocal<Value> Environment::BootstrapNode() {
   EscapableHandleScope scope(isolate_);
 
+  Local<Object> global = context()->Global();
+  // TODO(joyeecheung): this can be done in JS land now.
+  global->Set(context(), FIXED_ONE_BYTE_STRING(isolate_, "global"), global)
+      .Check();
+
   // process, require, internalBinding, primordials
   std::vector<Local<String>> node_params = {
       process_string(),
@@ -352,7 +352,7 @@ MaybeLocal<Value> Environment::BootstrapNode() {
       primordials_string()};
   std::vector<Local<Value>> node_args = {
       process_object(),
-      builtin_module_require(),
+      native_module_require(),
       internal_binding_loader(),
       primordials()};
 
@@ -441,10 +441,10 @@ MaybeLocal<Value> StartExecution(Environment* env, const char* main_script_id) {
 
   std::vector<Local<Value>> arguments = {
       env->process_object(),
-      env->builtin_module_require(),
+      env->native_module_require(),
       env->internal_binding_loader(),
       env->primordials(),
-      NewFunctionTemplate(env->isolate(), MarkBootstrapComplete)
+      env->NewFunctionTemplate(MarkBootstrapComplete)
           ->GetFunction(env->context())
           .ToLocalChecked()};
 
@@ -466,21 +466,13 @@ MaybeLocal<Value> StartExecution(Environment* env, StartExecutionCallback cb) {
       return {};
 
     StartExecutionCallbackInfo info = {
-        env->process_object(),
-        env->builtin_module_require(),
+      env->process_object(),
+      env->native_module_require(),
     };
 
     return scope.EscapeMaybe(cb(info));
   }
 
-  // TODO(joyeecheung): move these conditions into JS land and let the
-  // deserialize main function take precedence. For workers, we need to
-  // move the pre-execution part into a different file that can be
-  // reused when dealing with user-defined main functions.
-  if (!env->snapshot_deserialize_main().IsEmpty()) {
-    return env->RunSnapshotDeserializeMain();
-  }
-
   if (env->worker_context() != nullptr) {
     return StartExecution(env, "internal/main/worker_thread");
   }
@@ -494,10 +486,6 @@ MaybeLocal<Value> StartExecution(Environment* env, StartExecutionCallback cb) {
     return StartExecution(env, "internal/main/inspect");
   }
 
-  if (per_process::cli_options->build_snapshot) {
-    return StartExecution(env, "internal/main/mksnapshot");
-  }
-
   if (per_process::cli_options->print_help) {
     return StartExecution(env, "internal/main/print_help");
   }
@@ -516,14 +504,6 @@ MaybeLocal<Value> StartExecution(Environment* env, StartExecutionCallback cb) {
     return StartExecution(env, "internal/main/check_syntax");
   }
 
-  if (env->options()->test_runner) {
-    return StartExecution(env, "internal/main/test_runner");
-  }
-
-  if (env->options()->watch_mode && !first_argv.empty()) {
-    return StartExecution(env, "internal/main/watch_mode");
-  }
-
   if (!first_argv.empty() && first_argv != "-") {
     return StartExecution(env, "internal/main/run_main_module");
   }
@@ -823,13 +803,6 @@ int ProcessGlobalArgs(std::vector<std::string>* args,
     return 12;
   }
 
-  // TODO(aduh95): remove this when the harmony-import-assertions flag
-  // is removed in V8.
-  if (std::find(v8_args.begin(), v8_args.end(),
-                "--no-harmony-import-assertions") == v8_args.end()) {
-    v8_args.emplace_back("--harmony-import-assertions");
-  }
-
   auto env_opts = per_process::cli_options->per_isolate->per_env;
   if (std::find(v8_args.begin(), v8_args.end(),
                 "--abort-on-uncaught-exception") != v8_args.end() ||
@@ -980,13 +953,13 @@ int InitializeNodeWithArgs(std::vector<std::string>* argv,
   }
 # endif
 
-#endif  // defined(NODE_HAVE_I18N_SUPPORT)
+#endif
 
-  builtins::BuiltinEnv::InitializeCodeCache();
+  NativeModuleEnv::InitializeCodeCache();
 
   // We should set node_is_initialized here instead of in node::Start,
   // otherwise embedders using node::Init to initialize everything will not be
-  // able to set it and native addons will not load for them.
+  // able to set it and native modules will not load for them.
   node_is_initialized = true;
   return 0;
 }
@@ -995,18 +968,6 @@ InitializationResult InitializeOncePerProcess(int argc, char** argv) {
   return InitializeOncePerProcess(argc, argv, kDefaultInitialization);
 }
 
-#if HAVE_OPENSSL
-InitializationResult handle_openssl_error(int exit_code,
-                                          const char* msg,
-                                          InitializationResult* result) {
-  result->exit_code = exit_code;
-  result->early_return = true;
-  fprintf(stderr, "%s", msg);
-  ERR_print_errors_fp(stderr);
-  return *result;
-}
-#endif
-
 InitializationResult InitializeOncePerProcess(
   int argc,
   char** argv,
@@ -1019,7 +980,7 @@ InitializationResult InitializeOncePerProcess(
 
   // Initialized the enabled list for Debug() calls with system
   // environment variables.
-  per_process::enabled_debug_list.Parse();
+  per_process::enabled_debug_list.Parse(nullptr);
 
   atexit(ResetStdio);
 
@@ -1086,6 +1047,7 @@ InitializationResult InitializeOncePerProcess(
     }
     // In the case of FIPS builds we should make sure
     // the random source is properly initialized first.
+#if OPENSSL_VERSION_MAJOR >= 3
     // Call OPENSSL_init_crypto to initialize OPENSSL_INIT_LOAD_CONFIG to
     // avoid the default behavior where errors raised during the parsing of the
     // OpenSSL configuration file are not propagated and cannot be detected.
@@ -1095,88 +1057,51 @@ InitializationResult InitializeOncePerProcess(
     // fipsinstall command. If the path to this file is incorrect no error
     // will be reported.
     //
-    // For Node.js this will mean that CSPRNG() will be called by V8 as
-    // part of its initialization process, and CSPRNG() will in turn call
-    // call RAND_status which will now always return 0, leading to an endless
-    // loop and the node process will appear to hang/freeze.
-
-    // Passing NULL as the config file will allow the default openssl.cnf file
-    // to be loaded, but the default section in that file will not be used,
-    // instead only the section that matches the value of conf_section_name
-    // will be read from the default configuration file.
-    const char* conf_file = nullptr;
-    // To allow for using the previous default where the 'openssl_conf' appname
-    // was used, the command line option 'openssl-shared-config' can be used to
-    // force the old behavior.
-    if (per_process::cli_options->openssl_shared_config) {
-      conf_section_name = "openssl_conf";
-    }
-    // Use OPENSSL_CONF environment variable is set.
+    // For Node.js this will mean that EntropySource will be called by V8 as
+    // part of its initialization process, and EntropySource will in turn call
+    // CheckEntropy. CheckEntropy will call RAND_status which will now always
+    // return 0, leading to an endless loop and the node process will appear to
+    // hang/freeze.
     std::string env_openssl_conf;
     credentials::SafeGetenv("OPENSSL_CONF", &env_openssl_conf);
-    if (!env_openssl_conf.empty()) {
-      conf_file = env_openssl_conf.c_str();
-    }
-    // Use --openssl-conf command line option if specified.
-    if (!per_process::cli_options->openssl_config.empty()) {
-      conf_file = per_process::cli_options->openssl_config.c_str();
-    }
 
-    OPENSSL_INIT_SETTINGS* settings = OPENSSL_INIT_new();
-    OPENSSL_INIT_set_config_filename(settings, conf_file);
-    OPENSSL_INIT_set_config_appname(settings, conf_section_name);
-    OPENSSL_INIT_set_config_file_flags(settings,
-                                       CONF_MFLAGS_IGNORE_MISSING_FILE);
+    bool has_cli_conf = !per_process::cli_options->openssl_config.empty();
+    if (has_cli_conf || !env_openssl_conf.empty()) {
+      OPENSSL_INIT_SETTINGS* settings = OPENSSL_INIT_new();
+      OPENSSL_INIT_set_config_file_flags(settings, CONF_MFLAGS_DEFAULT_SECTION);
+      if (has_cli_conf) {
+        const char* conf = per_process::cli_options->openssl_config.c_str();
+        OPENSSL_INIT_set_config_filename(settings, conf);
+      }
+      OPENSSL_init_crypto(OPENSSL_INIT_LOAD_CONFIG, settings);
+      OPENSSL_INIT_free(settings);
 
-    OPENSSL_init_crypto(OPENSSL_INIT_LOAD_CONFIG, settings);
-    OPENSSL_INIT_free(settings);
-
-#if OPENSSL_VERSION_MAJOR < 3
-    if (ERR_peek_error() != 0) {
-      int ossl_error_code = ERR_GET_REASON(ERR_peek_error());
-      if (ossl_error_code == EVP_R_FIPS_MODE_NOT_SUPPORTED) {
-        if (!crypto::ProcessFipsOptions()) {
-          return handle_openssl_error(ossl_error_code, fips_error_msg, &result);
-        }
-      } else {
-        return handle_openssl_error(ossl_error_code,
-                                    "OpenSSL configuration error:\n",
-                                    &result);
+      if (ERR_peek_error() != 0) {
+        result.exit_code = ERR_GET_REASON(ERR_peek_error());
+        result.early_return = true;
+        fprintf(stderr, "OpenSSL configuration error:\n");
+        ERR_print_errors_fp(stderr);
+        return result;
       }
     }
 #else
-    if (ERR_peek_error() != 0) {
-      return handle_openssl_error(ERR_GET_REASON(ERR_peek_error()),
-                                  "OpenSSL configuration error:\n",
-                                  &result);
-      }
-#endif
-
-#if OPENSSL_VERSION_MAJOR < 3
     if (FIPS_mode()) {
       OPENSSL_init();
     }
 #endif
-
-    if (!crypto::ProcessFipsOptions()) {
-      return handle_openssl_error(ERR_GET_REASON(ERR_peek_error()),
-                                  fips_error_msg,
-                                  &result);
-    }
-
-    // Ensure CSPRNG is properly seeded.
-    CHECK(crypto::CSPRNG(nullptr, 0).is_ok());
-
-    V8::SetEntropySource([](unsigned char* buffer, size_t length) {
-      // V8 falls back to very weak entropy when this function fails
-      // and /dev/urandom isn't available. That wouldn't be so bad if
-      // the entropy was only used for Math.random() but it's also used for
-      // hash table and address space layout randomization. Better to abort.
-      CHECK(crypto::CSPRNG(buffer, length).is_ok());
-      return true;
-    });
-#endif  // HAVE_OPENSSL && !defined(OPENSSL_IS_BORINGSSL)
+  if (!crypto::ProcessFipsOptions()) {
+      result.exit_code = ERR_GET_REASON(ERR_peek_error());
+      result.early_return = true;
+      fprintf(stderr, "OpenSSL error when trying to enable FIPS:\n");
+      ERR_print_errors_fp(stderr);
+      return result;
   }
+
+  // V8 on Windows doesn't have a good source of entropy. Seed it from
+  // OpenSSL's pool.
+  V8::SetEntropySource(crypto::EntropySource);
+#endif  // HAVE_OPENSSL && !defined(OPENSSL_IS_BORINGSSL)
+}
   per_process::v8_platform.Initialize(
       static_cast<int>(per_process::cli_options->v8_thread_pool_size));
   if (init_flags & kInitializeV8) {
@@ -1212,26 +1137,29 @@ int Start(int argc, char** argv) {
     return result.exit_code;
   }
 
-  if (per_process::cli_options->build_snapshot) {
-    fprintf(stderr,
-            "--build-snapshot is not yet supported in the node binary\n");
-    return 1;
-  }
-
   {
+    Isolate::CreateParams params;
+    const std::vector<size_t>* indices = nullptr;
+    const EnvSerializeInfo* env_info = nullptr;
     bool use_node_snapshot =
         per_process::cli_options->per_isolate->node_snapshot;
-    const SnapshotData* snapshot_data =
-        use_node_snapshot ? SnapshotBuilder::GetEmbeddedSnapshotData()
-                          : nullptr;
+    if (use_node_snapshot) {
+      v8::StartupData* blob = NodeMainInstance::GetEmbeddedSnapshotBlob();
+      if (blob != nullptr) {
+        params.snapshot_blob = blob;
+        indices = NodeMainInstance::GetIsolateDataIndices();
+        env_info = NodeMainInstance::GetEnvSerializeInfo();
+      }
+    }
     uv_loop_configure(uv_default_loop(), UV_METRICS_IDLE_TIME);
 
-    NodeMainInstance main_instance(snapshot_data,
+    NodeMainInstance main_instance(&params,
                                    uv_default_loop(),
                                    per_process::v8_platform.Platform(),
                                    result.args,
-                                   result.exec_args);
-    result.exit_code = main_instance.Run();
+                                   result.exec_args,
+                                   indices);
+    result.exit_code = main_instance.Run(env_info);
   }
 
   TearDownOncePerProcess();
